# Tkabber debugging procedures
# (c) Marshall T. Rose 

# Some tweaks by Badlop

# Example options:
# set debug_lvls	[list jlib roster plugins]
# set debug_winP	1

proc debug_setup_menu {} {
    global debug_modules debug_lvls
    set m [.mainframe getmenu plugins]
    set ind [$m index end]

    set buttons [menu $m.devel -tearoff $ifacetk::options(show_tearoffs)]
    $buttons add checkbutton -label [::msgcat::mc "Debug to window"] -variable debug_winP

    $buttons add separator
    foreach l [list avatar       \
                    browser      \
                    chat         \
                    conference   \
                    custom       \
		    disco        \
                    emoticons    \
                    filetransfer \
                    filters      \
                    georoster    \
                    hook         \
		    http         \
                    iface        \
                    iq           \
                    jidlink      \
                    jlib         \
                    jlib::route  \
                    logger       \
                    login        \
                    message      \
                    negotiate    \
                    nick         \
                    plugins      \
                    presence     \
                    register     \
                    roster       \
                    search       \
                    ssj          \
                    tkabber      \
                    userinfo     \
                    utils        \
                    warning      \
                    zerobot] {
        if {[lsearch -exact $debug_lvls $l] >= 0} {
            set debug_modules($l) 1
        } else {
            set debug_modules($l) 0
        }

	$buttons add checkbutton -label $l -variable debug_modules($l) \
	    -command [namespace current]::debug_update
    }

    $m add cascade -label [::msgcat::mc "Debug live"] -menu $buttons 
}
hook::add finload_hook [namespace current]::debug_setup_menu

proc debug_update {} {
    global debug_modules debug_lvls
    
    set debug_lvls {}
    foreach {k v} [array get debug_modules] {
        if {$v} {
            lappend debug_lvls $k
        }
    }
}

proc ::debugmsg {module msg} {
    global debug_lvls w
    global debug_fd debug_winP

    if {![info exists debug_fd]} {
        catch { file rename -force -- ~/.tkabber/tkabber.log \
                                      ~/.tkabber/tkabber0.log }
        set debug_fd [open ~/.tkabber/tkabber.log \
                           { WRONLY CREAT TRUNC APPEND }]
        fconfigure $debug_fd -buffering line
    }

    puts $debug_fd [format "%s %-12.12s %s" \
                           [clock format [clock seconds] -format "%m/%d %T"] \
                           $module $msg]

    if {([lsearch -exact $debug_lvls $module] < 0) || (!$debug_winP)} {
        return
    }

    set dw .debug

    if {![winfo exists $dw]} {
        add_win $dw -title Debug -tabtitle debug -class Chat \
	    -raisecmd "focus [list $dw.body]
		       tab_set_updated [list $dw]"

        [ScrolledWindow $dw.sw] setwidget \
            [text $dw.body -yscrollcommand [list $dw.scroll set] \
		 -state disabled -highlightthickness 0]

        pack $dw.sw -side bottom -fill both -expand yes

        $dw.body tag configure module \
            -foreground [option get $dw theyforeground Chat]
        $dw.body tag configure proc   \
            -foreground [option get $dw meforeground Chat]
        $dw.body tag configure error  \
            -foreground [option get $dw errforeground Chat]
    }

    $dw.body configure -state normal

    set scroll [expr {[lindex [$dw.body yview] 1] == 1}]

    $dw.body insert end [format "%s: %-12.12s" [clock format [clock seconds] -format "%m/%d %T"] \
	$module] module " "
    set tag normal
    switch -- $module {
        jlib {
            if {[set x [string first "(jlib::" $msg]] > 0} {
                set tag error
            }
            if {[set y [string first ")" $msg]] > 0} {
                $dw.body insert end \
                         [string range $msg [expr $x+7] [expr $y-1]] proc \
                         "\n"
                set msg [string trimleft \
                                [string range $msg [expr $y+1] end]]
            }
        }

        default {
        }
    }
    $dw.body insert end [string trimright $msg] $tag
    $dw.body insert end "\n\n"

    if {!$chat::options(stop_scroll) && \
	    (!$chat::options(smart_scroll) || \
		($chat::options(smart_scroll) && $scroll))} {
        $dw.body see end
    }

    $dw.body configure -state disabled
}

#namespace eval zerobot {
#    proc examine_message {from id type subject body err thread priority x} {
#        global userstatus
#
#       switch -- $userstatus {
#           away
#               -
#           xa {
#               if {([string length [set body [checkP $from $type $subject \
#                                                    $body $x]]] > 0) \
#                       && ([catch { notify $from $body } result])} {
#                   debugmsg zerobot "notify: $result"
#               }
#           }
#           default {
#               reset
#           }
#       }
#   }
#   hook::add examine_message_hook   [namespace current]::examine_message   10
#}
